
/*
 * Name: Bhaskar S
 * 
 * Date: 03/07/2009 
 */

package com.polarsparc.restlet.example.two;

import org.restlet.*;
import org.restlet.data.*;
import org.restlet.resource.*;

// Resource represents an abstract web entity that can be identified
// with a URL. A Resource also manages data representations
public class DvdResource extends Resource {
	private static DvdDAO _DAO;
	
	static {
		try {
			_DAO = new DvdDAO();
			_DAO.init();
		}
		catch (Exception ex) {
			ex.printStackTrace(System.err);
			System.exit(1);
		}
	}
	
	public DvdResource(Context context, Request request, Response response) {  
		super(context, request, response);  

		// Variant is an abstraction of data representation for a Resource
		// In this case we are only supporting XML data representation
		getVariants().add(new Variant(MediaType.TEXT_XML));  
	}
	
	@Override
	// Important - To handle POST
	public boolean allowPost() {
		return true;
	}
	
	@Override
	// Important - To handle PUT
	public boolean allowPut() {
		return true;
	}
	
	@Override
	// Important - To handle DELETE
	public boolean allowDelete() {
		return true;
	}
	
	@Override
	// Handle GET - Need to override to implement the state behind
	// our DVD resource
	public Representation represent(Variant variant)
		throws ResourceException {
		if (MediaType.TEXT_XML.equals(variant.getMediaType())) {
			String movie = (String) getRequest().getAttributes().get("movie");
			if (movie == null || movie.trim().length() == 0) {
				getResponse().setStatus(Status.CLIENT_ERROR_BAD_REQUEST);
				return new StringRepresentation("<Error>No movie name specified for query</Error>", MediaType.TEXT_XML);
			}
			
			System.out.println("-> Requested DVD Movie: " + movie);
			
			String info = null;
			
			try {
				info = _DAO.find(movie);
			}
			catch (Exception ex) {
				ex.printStackTrace(System.err);
			}
			
			if (info != null) {
				return new StringRepresentation(info, MediaType.TEXT_XML);
			}
		}
		return null;
	}
	
	@Override  
	// Handle POST - Need to override to add a new DVD resource
	public void acceptRepresentation(Representation entity)  
		throws ResourceException {
		String movie = (String) getRequest().getAttributes().get("movie");
		if (movie == null || movie.trim().length() == 0) {
			getResponse().setStatus(Status.CLIENT_ERROR_BAD_REQUEST);
			getResponse().setEntity(new StringRepresentation("<Error>No movie name specified for create</Error>", MediaType.TEXT_XML));
			return;
		}
		
		if (entity.getMediaType().equals(MediaType.APPLICATION_WWW_FORM, true)) {
		    // Retrieve name-value pairs from the Form  
		    Form form = new Form(entity);
		    
		    String name = form.getFirstValue("name");
		    String genre = form.getFirstValue("genre");  
		    String lang = form.getFirstValue("language");
		    String format = form.getFirstValue("format");
		    String year = form.getFirstValue("year");
		    String prod = form.getFirstValue("producer");
		    String price = form.getFirstValue("price");
		    
		    boolean status = false;
			
			try {
				status = _DAO.update(movie, name, genre, lang, format, year, prod, price);
			}
			catch (Exception ex) {
				ex.printStackTrace(System.err);
			}
			
			if (status) {
				getResponse().setStatus(Status.SUCCESS_CREATED);
				getResponse().setEntity(new StringRepresentation("<Success>DVD information created for movie " + movie + "</Success>", MediaType.TEXT_XML));
			}
			else {
				getResponse().setStatus(Status.CLIENT_ERROR_BAD_REQUEST);
				getResponse().setEntity(new StringRepresentation("<Error>Could not create DVD information for movie " + movie + "</Error>", MediaType.TEXT_XML));
			}
		}
		else {
			getResponse().setStatus(Status.CLIENT_ERROR_BAD_REQUEST);
			getResponse().setEntity(new StringRepresentation("<Error>Invalid request to create DVD information for movie " + movie + "</Error>", MediaType.TEXT_XML));
		}
	}
	
	@Override  
	// Handle PUT - Need to override to update existing DVD resource
	public void storeRepresentation(Representation entity)  
		throws ResourceException {
		String movie = (String) getRequest().getAttributes().get("movie");
		if (movie == null || movie.trim().length() == 0) {
			getResponse().setStatus(Status.CLIENT_ERROR_BAD_REQUEST);
			getResponse().setEntity(new StringRepresentation("<Error>No movie name specified for update</Error>", MediaType.TEXT_XML));
			return;
		}
		
		if (entity.getMediaType().equals(MediaType.APPLICATION_WWW_FORM, true)) {
		    // Retrieve name-value pairs from the Form  
		    Form form = new Form(entity);
		    
		    String name = form.getFirstValue("name");
		    String genre = form.getFirstValue("genre");  
		    String lang = form.getFirstValue("language");
		    String format = form.getFirstValue("format");
		    String year = form.getFirstValue("year");
		    String prod = form.getFirstValue("producer");
		    String price = form.getFirstValue("price");
		    
		    boolean status = false;
			
			try {
				status = _DAO.update(movie, name, genre, lang, format, year, prod, price);
			}
			catch (Exception ex) {
				ex.printStackTrace(System.err);
			}
			
			if (status) {
				getResponse().setStatus(Status.SUCCESS_CREATED);
				getResponse().setEntity(new StringRepresentation("<Success>DVD information updated for movie " + movie + "</Success>", MediaType.TEXT_XML));
			}
			else {
				getResponse().setStatus(Status.CLIENT_ERROR_BAD_REQUEST);
				getResponse().setEntity(new StringRepresentation("<Error>Could not update DVD information for movie " + movie + "</Error>", MediaType.TEXT_XML));
			}
		}
		else {
			getResponse().setStatus(Status.CLIENT_ERROR_BAD_REQUEST);
			getResponse().setEntity(new StringRepresentation("<Error>Invalid request to update DVD information for movie " + movie + "</Error>", MediaType.TEXT_XML));
		}
	}
	
	@Override  
	// Handle DELETE - Need to override to delete existing DVD resource
	public void removeRepresentations()
		throws ResourceException {
		String movie = (String) getRequest().getAttributes().get("movie");
		if (movie == null || movie.trim().length() == 0) {
			getResponse().setStatus(Status.CLIENT_ERROR_BAD_REQUEST);
			getResponse().setEntity(new StringRepresentation("<Error>No movie name specified for delete</Error>", MediaType.TEXT_XML));
			return;
		}
	    
	    boolean status = false;
		
		try {
			status = _DAO.delete(movie);
		}
		catch (Exception ex) {
			ex.printStackTrace(System.err);
		}
		
		if (status) {
			getResponse().setStatus(Status.SUCCESS_CREATED);
			getResponse().setEntity(new StringRepresentation("<Success>DVD information deleted for movie " + movie + "</Success>", MediaType.TEXT_XML));
		}
		else {
			getResponse().setStatus(Status.CLIENT_ERROR_BAD_REQUEST);
			getResponse().setEntity(new StringRepresentation("<Error>Could not delete DVD information for movie " + movie + "</Error>", MediaType.TEXT_XML));
		}
	}
}